/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes

#include "lm48100q.h"
#include <math.h>
// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************
volatile float f_s = 44000;                                                     
volatile float two32 = 4294967296;                                              

volatile bool tmr_flag = false;                                                 
volatile float f_out = 300; 

volatile uint32_t N;
volatile uint32_t phase_accum;



const uint16_t sin_table[256] = {2048, 2098, 2148, 2199, 2249, 2299, 2349, 2398, 2448, 2497, 2546, 2594, 2643, 2690, 2738, 2785, 2832,
2878, 2924, 2969, 3013, 3057, 3101, 3144, 3186, 3227, 3268, 3308, 3347, 3386, 3423, 3460, 3496, 3531, 3565, 3599, 3631, 3663, 3693, 
3722, 3751, 3778, 3805, 3830, 3854, 3877, 3899, 3920, 3940, 3959, 3976, 3993, 4008, 4022, 4035, 4046, 4057, 4066, 4074, 4081, 4086, 
4090, 4094, 4095, 4095, 4095, 4094, 4090, 4086, 4081, 4074, 4066, 4057, 4046, 4035, 4022, 4008, 3993, 3976, 3959, 3940, 3920, 3899, 
3877, 3854, 3830, 3805, 3778, 3751, 3722, 3693, 3663, 3631, 3599, 3565, 3531, 3496, 3460, 3423, 3386, 3347, 3308, 3268, 3227, 3186, 
3144, 3101, 3057, 3013, 2969, 2924, 2878, 2832, 2785, 2738, 2690, 2643, 2594, 2546, 2497, 2448, 2398, 2349, 2299, 2249, 2199, 2148, 2098, 
2048, 1998, 1948, 1897, 1847, 1797, 1747, 1698, 1648, 1599, 1550, 1502, 1453, 1406, 1358, 1311, 1264, 1218, 1172, 1127, 1083, 1039, 995, 
952, 910, 869, 828, 788, 749, 710, 673, 636, 600, 565, 531, 497, 465, 433, 403, 374, 345, 318, 291, 266, 242, 219, 197, 176, 156, 137, 120, 
103, 88, 74, 61, 50, 39, 30, 22, 15, 10, 6, 2, 1, 0, 1, 2, 6, 10, 15, 22, 30, 39, 50, 61, 74, 88, 103, 120, 137, 156, 176, 197, 219, 242, 
266, 291, 318, 345, 374, 403, 433, 465, 497, 531, 565, 600, 636, 673, 710, 749, 788, 828, 869, 910, 952, 
995, 1039, 1083, 1127, 1172, 1218, 1264, 1311, 1358, 1406, 1453, 1502, 1550, 1599, 1648, 1698, 1747, 1797, 1847, 1897, 1948, 1998};

uint16_t dac_in;

void TMR2_IntHandler(uint32_t status, uintptr_t context){
    tmr_flag = true;
}

int main ( void )
{
    uint32_t i=0;
     N =(uint32_t)(f_out*(two32/f_s)+0.5);
    /* Initialize all modules */
    SYS_Initialize ( NULL );

    
    lm48100q_set_volume(10);
    
    TMR2_CallbackRegister(TMR2_IntHandler, (uintptr_t)NULL);
    TMR2_Start();
    TMR2_PeriodSet(2273);                                                       // Inkrement tajmera desava se na svakih 1/f_clk = 1/100 MHz = 10 ns --> Za 1/44000 = 22,73 us = 2273 * 10 ns.
    
    uint8_t data[2];
    while ( true )
    {
        if(tmr_flag){
            tmr_flag = false;
            
            if(i<=5720){                                                        // Od nultog do 5720. odbirka izlazna frekvencija se menja kao y = -260sin((-3.14/5720)*x) + 1740.
                f_out = -260*sin((-3.14/5720)*i) + 1740;                        
            }
            else if(i<=2*5720){                                                 // od 5721. do 11440. odbirka izlazna frekvencija se menja kao y = 1.53e-4 * x^2 + 2000.
                f_out = 1.53e-4 * pow(i,2) + 2000;
            }
            /*if(i<=8976){
                f_out = -3.7098E-19*pow(i,6)+ 9.64231E-15*pow(i,5)-9.09965E-11*pow(i,4) + 0.000000379*pow(i,3) - 0.000730447*pow(i,2)+0.839879*i+1207.82;
            }
            */
            else{ 
                break;
            }
   
            i++;
            N =(uint32_t)(f_out*(two32/f_s)+0.5);
            phase_accum+=N;
            dac_in = sin_table[phase_accum>>24]|0x7000;
            data[0] = dac_in>>8;
            data[1] = dac_in&0xFF;

            GPIO_PinClear(GPIO_PIN_RF0);
            SPI5_Write(data, 2);
            while(SPI5_IsTransmitterBusy());
            GPIO_PinSet(GPIO_PIN_RF0);

        }
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

